%
% (2) Computation and test of the Youla parameterization
%
clear;
%
% The open loop LFT models and the state-feedback
% and observer gains (K1, K2 and L) are loaded
%
load sf_obs_controller;
%
% Computation of the augmented controller
%
[a,b,c,d]=ssdata(sys_M_without_integrator);
nx=size(a,1);
N=sum(abs(blk(:,1)));
[ny,nu]=size(d);
ny=ny-N;
nu=nu-N;
%
idxO1=1:N;
idxO2=(N+1):(N+ny);
idxI1=1:N;
idxI2=(N+1):(N+nu);
b1=b(:,idxI1);
b2=b(:,idxI2);
c1=c(idxO1,:);
c2=c(idxO2,:);
d11=d(idxO1,idxI1);
d12=d(idxO1,idxI2);
d21=d(idxO2,idxI1);
d22=d(idxO2,idxI2);
if norm(d22) error('D22 SHOULD BE ZERO'); end
%
% Augmented controller  inputs: w_hat, y, alfa-alfa_r (SCALAR input for integrator), v
%                      outputs: z_hat, u, epsilon (observation error)
%
ak=[a-b2*K1-L*c2 -b2*K2; zeros(1,nx+1)];
bk=[b1-L*d21 L zeros(nx,1) b2; zeros(1,N+ny) 1 zeros(1,nu)];
ck=[c1-d12*K1 -d12*K2; -K1 -K2; -c2 zeros(ny,1)];
dk=[d11 zeros(N,ny+1) d12; zeros(nu,N+ny+1) eye(nu); -d21 eye(ny) zeros(ny,1) zeros(ny,nu)];
%
% First I/O of the closed loop: for the interconnection with Delta.
% Input 2*N+1: alfa_ref
% Input 2*N+2: delta u (additive perturbation on the feedback input)
% Input 2*N+3: v (for the interconnection with the Youla parameter)
% Outputs 2*N+1 to 2*N+4: q, alfa, actuator position, actuator rate output.
% Outputs 2*N+5: alfa-alfa_r.
% Ouputs 2*N+6 to 2*N+7: epsilon (observation error - for the interconnection
%                                 with the Youla parameter)
%
[a,b,c,d]=linmod('closed_loop_youla');
sys_cl_aug=ss(a,b,c,d);
%
% The first I/O are re-ordered, so as to gather the parametric uncertainties
% in Delta_aug=diag(Delta,Delta)
%
idx=[];
for i=1:N
  idx=[idx i i+N];
end
idxI=[idx (2*N+1):size(d,2)];
idxO=[idx (2*N+1):size(d,1)];
sys_cl_aug=sys_cl_aug(idxO,idxI);
blk_aug=2*blk;
%
% Standard form for the LFT design
%
youla=[];
%
youla.blk=blk_aug;
%
% To define youla.idxI1, youla.idxI2, youla.idxO1 and youla.idxO2
% forget the first I/O of the closed loop LFT model, corresponding
% to the interconnection with Delta_aug.
%
youla.idxI1=1:2; % alfa_ref and delta_u
youla.idxI2=3;   % v
youla.idxO1=1:5; % q, alfa, actuator position, actuator rate output, alfa-alfa_r
youla.idxO2=6:7; % epsilon
%
[a,b,c,d]=ssdata(sys_cl_aug);
N=sum(abs(blk_aug(:,1)));
idxI1=1:N;
idxI2=(N+1):size(d,2);
idxO1=1:N;
idxO2=(N+1):size(d,1);
youla.sys_lft=[];
youla.sys_lft.A=a;
youla.sys_lft.B1=b(:,idxI1);
youla.sys_lft.B2=b(:,idxI2);
youla.sys_lft.C1=c(idxO1,:);
youla.sys_lft.C2=c(idxO2,:);
youla.sys_lft.D11=d(idxO1,idxI1);
youla.sys_lft.D12=d(idxO1,idxI2);
youla.sys_lft.D21=d(idxO2,idxI1);
youla.sys_lft.D22=d(idxO2,idxI2);
%
fprintf('\n\n The Youla parameterization has been saved in youla.mat\n');
%
save youla youla;
%
%
%
fprintf('\n Does the closed loop have the form T1(s,Delta) + T2(s,Delta) Q(s) T3(s,Delta) ? The error below should be zero \n');
%
n_delta=size(blk_aug,1);
tab_delta=[zeros(1,n_delta); rand(1,n_delta)];
%
youla_at_one_point=[];
youla_at_one_point.idxI1=youla.idxI1;
youla_at_one_point.idxI2=youla.idxI2;
youla_at_one_point.idxO1=youla.idxO1;
youla_at_one_point.idxO2=youla.idxO2;
%
puls=linspace(0,1e2,200);
list_poles=[-1 -2 -3];
%
nu=length(youla.idxI2);
ny=length(youla.idxO2);
sys_Q0=ss([],[],[],zeros(nu,ny));
tab_sys_Q=gene_basis(list_poles,nu,ny);
Nbr_filters=length(tab_sys_Q);
%
tuning=[];
tuning.amp=1;
%
for i=1:size(tab_delta,1)
  Delta=[];
  for j=1:n_delta
    nj=abs(blk_aug(j,1));
    Delta=mdiag(Delta,tab_delta(i,j)*eye(nj));
  end
  youla_at_one_point.sys_T=calc_lft_at_one_point(youla.sys_lft,Delta);
  comp_freq_responses(puls,[],youla_at_one_point,sys_Q0,tab_sys_Q,1,1,tuning);
  pause_talk;
end
