function [specif,puls_specif]=design_spec(Gain_roll_off,omega_roll_off)
%
% call:  [specif,puls_specif]=design_spec(Gain_roll_off,omega_roll_off)
%
% Computation of the design specifications
%
% Gain_roll_off and omega_roll_off define the Hinfinity constraint
%
if nargin~=2 error('DESIGN_SPEC: 2 input arguments'); end
%
puls_specif=[5 25 0 logspace2(1e-2,1e3,100)];
%
specif=[];
%
% The I/O of T1 + T2 Q T3 are:
% Inputs  1 to 2  : additive perturbations on the plant inputs used by the feedback controller
% Outputs 1 to 4  : ny,p,r,phi
%         5 to 6  : rigid and flexible ny.
%         7 to 8  : actuator rates.
%         9       : beta.
%        10 to 11 : feedback controller outputs.
%

%
% Hinf minimization of the transfer function between the 1st disturbance input and ny
%
specif{1}.input=1;
specif{1}.output=1;
specif{1}.list=[1 2];
specif{1}.template=1e-3*[1 1];
specif{1}.obj=1;
specif{1}.lin_log_x=0;
specif{1}.lin_log_y=0;
specif{1}.interp=0;
%
% Hinf minimization of the transfer function between the 2d disturbance input and ny
%
specif{2}.input=2;
specif{2}.output=1;
specif{2}.list=[1 2];
specif{2}.template=1e-3*[1 1];
specif{2}.obj=1;
specif{2}.lin_log_x=0;
specif{2}.lin_log_y=0;
specif{2}.interp=0;
%
% Hinf constraint between the two disturbance inputs and the two feedback controller outputs
%
num=Gain_roll_off;
den=conv([1/omega_roll_off 1],[1/omega_roll_off/omega_roll_off 1.4/omega_roll_off 1]);
[a,b,c,d]=tf2ss(num,den);
sys=ss(a,b,c,d);
%
specif{3}.input=[1 2];
specif{3}.output=[10 11];
specif{3}.list=3:length(puls_specif);
specif{3}.template=squeeze(abs(freqresp(sys,puls_specif(3:end))));
specif{3}.obj=0;
specif{3}.lin_log_x=1;
specif{3}.lin_log_y=1;
specif{3}.interp=1;
