%
% call:  [K,OK]=calc_state_feedback_obs_lft_one_shot(sys_M,blk,sec_pol,tab_delta,tol_P,0)
%        [L,OK]=calc_state_feedback_obs_lft_one_shot(sys_M,blk,sec_pol,tab_delta,tol_P,1)
%
% One-shot multi-model design of a state-feedback gain K or observer gain L for an LFT model.
%
% Use of the LMI Control Toolbox.
%
% The plant model inside the observer is the LFT model itself, i.e. the
% uncertainties are supposed to be measured, but the state-feedback and observer
% gains do not depend on the uncertainties.
%
% The issue is to ensure modal specifications for all structured model uncertainties Delta
% defined by tab_delta and blk.
%
% The LFT model is F_u(M(s),Delta). The first I/O of M(s) are connected to Delta.
% The last I/O are physical ones, that are ALL used by the feedback controller.
% The plant model between the physical inputs and outputs is assumed to be strictly proper.
%
% sys_M = state-space model of M(s).
%
% The structure of Delta is described by blk (musyn format).
%
% tab_delta(i,:) = value # i of the parametric uncertainties used for the design.
%
% sec_pol = [lambda r cxi] describes the requirements for the closed loop poles:
%
% cxi = minimal damping ratio. cxi >=0 is required.
% lambda = degree of stability = opposite of the maximal value of the poles
% real part. lambda>=0 is required (stable system).
% r = maximal magnitude of the poles. r > 0 is required.
% r = Inf is possible.
% r = NaN means that r is to be minimized on line.
%
% tol_P > 0 is required.
% tol_P = 0 is often possible but this may enlarge the value of K.
% Choose tol_P = [] to choose the default value.
%
% A too large value of tol_P may make the LMIs unfeasible:
% if the multi-model design fails please check if the value of tol_P is not too large !
%
% tol_P may be used as a last tuning parameter, to decrease the norm of K.
%
% sf_obs = 0: design of a state-feedback gain K.
% The closed loop state-matrix is A(delta)-B(delta)*K,
% where delta is the vector of parametric uncertainties.
%
% sf_obs = 1: design of an observer gain L.
% The closed loop state-matrix is A(delta)-L*C(delta).
%
% OK = 0 and K = [] means that the multi-model design failed.
%
