%
% call:  [sys_Q,alfa_opt,OK_design,subgradients]=design_convex(youla,sys_Q0,tab_sys_Q,tab_nbr_variables,...
%        tuning,puls_syn,specif,puls_specif,puls_valid,is_linearity_test,first_figure)
%
%        [sys_Q,alfa_opt,OK_design,subgradients]=design_convex(youla,sys_Q0,tab_sys_Q,tab_nbr_variables,...
%        tuning,puls_syn,specif,puls_specif,puls_valid,is_linearity_test,first_figure,subgradients)
%
% Convex design of a feedback controller.
%
% youla.sys_T defines the closed loop transfer matrix T(s) = [T1(s) T2(s); T3(s) 0],
% so that the feedback interconnection with the Youla parameter Q(s) is T1(s) + T2(s) Q(s) T3(s).
%
% youla.idxI2 and youla.idxO2 are the lists of inputs and outputs of T(s), corresponding to the
% feedback with Q(s).
% youla.idxI1 and youla.idxO1 are the lists of inputs and outputs of T(s), corresponding to the
% design specifications.
%
% Q(s) = Q_0(s) + \sum_i \theta_i Q_i(s), where the \theta_i are the optimisation parameters.
% sys_Q0 is a state-space representation of  Q_0(s).
% tab_sys_Q{i} is a state-space representation of Q_i(s).
%
% tab_nbr_variables is a row vector containing an increasing number of optimisation variables.
% If tab_nbr_variables = [10 20 30], then the optimisation is performed w.r.t. the first 10
% optimisation variables, then w.r.t. to the first 20 ones, and finally w.r.t. to all 30 ones.
% figure(2000) is produced if length(tab_nbr_variables) > 1.
% For a one-shot design, please choose tab_nbr_variables=length(tab_sys_Q);
%
% see optim_new.m for "tuning", noting that 2 additional parameters are added:
% * tuning.tol_constraint: tolerance on the constraints.
% (see options.tol1 in validation.m)
% * tuning.zero_T4 = 0/1: if it is set to 1, T4(s) is set to zero during the validation step.
%
% puls_syn is the initial design frequency gridding (that will be refined inside the routine).
%
% specif and puls_specif describe the specifications.
% Remark: the case of H2 minimization without any Hinf spec. is not allowed.
%
% puls_valid is a fine frequency gridding for validation and H2 specifications.
%
% is_linearity_test = 0 --> No test of the linearity of T1 + T2 Q T3
%                     1 --> One preliminary test
%                     2 --> One preliminary test + On-line tests with the optimal controller
%
% Indeed, youla.sys_T is a state-space representation without any structure,
% i.e. the zero term in T(s) = [T1(s) T2(s); T3(s) 0] is not explicit.
% The closed loop transfer matrix with the Youla parameter Q(s) is computed
% as the feedback interconnection of T(s) with Q(s).
% If T(s) = [T1(s) T2(s); T3(s) T4(s)] with a non-zero T4(s), there will be a problem.
% See  also the help of comp_freq_responses.m
%
% first_figure is the number of the first figure produced by the routine.
%
% No graphical display if first_figure=0, except possibly on figure(2000)
% in the case of a progressive design.
%
% See optim_new.m for "subgradients". This variable can be used to save much computational time
% when performing numerous computations of the Youla parameter with the same design specifications,
% i.e. when studying a trade-off between the spec.
%
% sys_Q is the optimal value of Q(s).
%
% alfa_opt is the minimized objective.
%
% OK_design = 1 / 0 --> success or not of the optimisation.
%
% Main subroutines (not included): comp_freq_responses.m, optim_new.m, validation.m
%
